import pandas as pd
import statsmodels.tsa.stattools as ts

from .pair import Pair


def cadf_test(pair):
    """
    Conducts a Cointegrated Augmented Dickey Fuller Test on a pair of equities.

    Prints t-statistic, p-value and critical values

    Parameters
    ----------
    pair : radium.Pair

    Raises
    ------
    TypeError
        If pair is not of type radium.Pair.
    """

    if not isinstance(pair, Pair):
        raise TypeError('Pair must be of type radium.Pair')

    # Get closed prices in dataframe
    equity1_df = pair.equity1.closed
    equity2_df = pair.equity2.closed

    # Set column name as symbol
    equity1_df = equity1_df.rename(pair.equity1.symbol)
    equity2_df = equity2_df.rename(pair.equity2.symbol)

    # Augment dataframes, removing missing data
    df = pd.concat([equity1_df, equity2_df], axis=1, join="inner")

    # Get CADF result
    coint_t, pvalue, crit_value = ts.coint(df[pair.equity1.symbol],
                                           df[pair.equity2.symbol])

    # Round to make more interpretable
    coint_t = round(coint_t, 3)
    pvalue = round(pvalue, 3)
    crit_value = [round(x, 3) for x in crit_value]

    # Print results
    results = (f'CADF Test for cointegration between {pair.equity1.symbol} '
               f'and {pair.equity2.symbol} from {pair.start_date} to '
               f'{pair.end_date}\n\n'
               f't-statistic = {coint_t}\n'
               f'p-value = {pvalue}\n\n'
               f'1% Critical Value: {crit_value[0]}\n'
               f'5% Critical Value: {crit_value[1]}\n'
               f'10% Critical Value: {crit_value[2]}\n\n')

    print(results)



from datetime import date as datetime_date
from datetime import datetime


def _convert_date(date):
    """
    Converts a date to datetime.date type

    Parameters
    ----------
    date : str or datetime or datetime.date
        First date to plot in YYYY-MM-DD form

    Returns
    -------
    date : datetime.date
        date as datetime.date type

    Raises
    ------
    TypeError
        Date not of types str or datetime or datetime.date
    """

    # Converts strings
    if isinstance(date, str):
        date = datetime.strptime(date, "%Y-%m-%d").date()

    # Converts date object
    elif isinstance(date, datetime):
        date = date.date()

    # If not in required form input is incorrect
    if not isinstance(date, datetime_date):
        raise TypeError("date should be a string or datetime object")

    return date



from .pair_strategy import PairStrategy
import numpy as np
import statsmodels.tsa.stattools as ts
import pandas as pd


class BollingerPair(PairStrategy):
    """
    Bollinger band strategy on a pair of equities

    Attributes
    ----------
    pair: radium.Pair
    entry_z : float
        Z-score to enter position at
    exit_z: float
        Z-score to exit position at
    lookback: int
        Days looked back at on when calculating optimal positions
    th_positions : np.float[][2]
        Theoretical optimum positions calculated by strategy

    See Also
    --------
    radium.PairStrategy : Parent class
    """

    def __init__(self, pair, entry_z, exit_z, lookback):
        """
        Initialises strategy

        Parameters
        ----------
        pair: radium.Pair
        entry_z : float
            Z-score to enter position at
        exit_z : float
            Z-score to exit position at
        lookback : int
            Days to lookback on when calculating optimal positions
        """

        super().__init__(pair)

        self.entry_z = entry_z
        self.exit_z = exit_z
        self.lookback = lookback
        self.th_positions = self.calculate_positions()


    def calculate_positions(self):

        """
        Calculates optimum positions calculated by strategy

        Returns
        -------
        th_positions : np.float[][2]
        """

        spread = self.pair.price_spread

        # Calculate Z-score
        spread_mean = spread.rolling(self.lookback).mean()
        spread_std = spread.rolling(self.lookback).std()
        spread_z = (spread - spread_mean) / spread_std

        # Calculate long and short entry and exit points
        long_entry = spread_z < -self.entry_z
        long_exit = spread_z > -self.exit_z

        short_entry = spread_z > self.entry_z
        short_exit = spread_z < self.exit_z

        # Calculate number of units at each point
        units_long = np.zeros(long_entry.shape)
        units_long[:] = np.nan

        units_short = np.zeros(short_entry.shape)
        units_short[:] = np.nan

        units_long[0] = 0
        units_long[long_entry] = 1
        units_long[long_exit] = 0
        units_long = pd.DataFrame(data=units_long,
                                  index=self.pair.equity1.data.index,
                                  columns=["Units Long"])
        units_long.fillna(method='ffill', inplace=True)

        units_short[0] = 0
        units_short[short_entry] = -1
        units_short[short_exit] = 0
        units_short = pd.DataFrame(data=units_short,
                                   index=self.pair.equity1.data.index,
                                   columns=["Units Short"])
        units_short.fillna(method='ffill', inplace=True)

        units = units_short.values + units_long.values

        # Convert units into a n * 2 matrix (due to pair of equities)
        units = np.tile(units, [1, 2])

        # Optimal share positions determined by the Bollinger Bands strategy
        return units * self.pair.hedge_ratios



import numpy as np
import pandas as pd
from radium.helpers import _truncate


def returns(self):
    """
    Calculates returns accounting for budget and commission

    Returns: Returns as 2D array
    """

    # Get the optimal positions determined by the strategy
    optimal_positions = self.positions()

    # Calculate integer positions determined by rounding optimal positions to 3 d.p.
    rounded_positions = np.zeros(optimal_positions.shape)
    for i in range(self.lookback, optimal_positions.shape[0] - 1):
        rounded_positions[i, 0] = _truncate(optimal_positions[i, 0],
                                            3) * 10 ** 3
        rounded_positions[i, 1] = _truncate(optimal_positions[i, 1],
                                            3) * 10 ** 3

    # Get closed prices
    prices = pd.concat([self.pair.equity1.closed, self.pair.equity2.closed],
                       axis=1)

    # Calculate orders
    equity1_orders = np.diff(rounded_positions[:, 0])
    equity1_orders = np.append(0, equity1_orders)
    equity2_orders = np.diff(rounded_positions[:, 1])
    equity2_orders = np.append(0, equity2_orders)

    # Calculate commissions per daily order (minimum price of 0.35)
    equity1_comm = np.array(
        [max(abs(x) * 0.0035, 0.35) if x != 0 else 0 for x in equity1_orders])
    equity2_comm = np.array(
        [max(abs(x) * 0.0035, 0.35) if x != 0 else 0 for x in equity2_orders])

    # Calculate Initial Budget, equal to buy 1000 units of each equity
    init_budget = 1000 * (self.pair.equity1.closed.iloc[0] +
                          self.pair.equity2.closed.iloc[0])
    # Truncate to 2 d.p.
    budget = _truncate(init_budget, 2)

    # Calculate returns from Equity 1 and Equity 2
    for i in range(0, len(equity1_orders) - 1):
        budget += -1 * equity1_orders[i] * self.pair.equity1.closed.iloc[i] - \
                  equity1_comm[i]
        budget += -1 * equity2_orders[i] * self.pair.equity2.closed.iloc[i] - \
                  equity2_comm[i]

    return budget / init_budget - 1



import math


def _truncate(number, decimals=0):
    """
    Truncates a number to a certain number of decimal places

    Parameters
    ----------
    number : numeric
        Number to truncate
    decimals : int
        Decimal places to truncate to, must be non-negative

    Returns
    -------
    ret : numeric
        Number truncated to specified decimal places

    Raises
    ------
    TypeError
        Decimals is not an integer
    ValueError
        Decimals is negative
    """

    if not isinstance(decimals, int):
        raise TypeError('Decimals must be an integer.')
    elif decimals < 0:
        raise ValueError('Decimals must be >= 0.')
    elif decimals == 0:
        # If decimals is zero can truncate as normal
        return math.trunc(number)

    factor = 10.0 ** decimals
    return math.trunc(number * factor) / factor



import statsmodels.tsa.vector_ar.vecm as vm
import pandas as pd

from .pair import Pair

def johansen_test(pair):
    """
    Conducts a Johansen Test on a pair of equities.

    Print trace/eigenvalue statistics and critical values.

    Parameters
    ----------
    pair : radium.Pair

    Raises
    ------
    TypeError
        If pair is not of type radium.Pair.
    """

    if not isinstance(pair, Pair):
        raise TypeError('Pair must be of type radium.Pair')

    # Get closed prices in dataframe
    equity1_df = pair.equity1.closed
    equity2_df = pair.equity2.closed

    # Set column name as symbol
    equity1_df = equity1_df.rename(pair.equity1.symbol)
    equity2_df = equity2_df.rename(pair.equity2.symbol)

    # Augment dataframes, removing missing data
    df = pd.concat([equity1_df, equity2_df], axis=1, join="inner")

    # Get Johansen results
    result = vm.coint_johansen(df.values, det_order=0, k_ar_diff=1)
    trace_stat = result.lr1
    trace_crit = result.cvt
    eigen_stat = result.lr2
    eigen_crit = result.cvm

    # Round to make more interpretable
    trace_stat = [round(x, 3) for x in trace_stat]
    trace_crit = [[round(x, 3) for x in y] for y in trace_crit]
    eigen_stat = [round(x, 3) for x in eigen_stat]
    eigen_crit = [[round(x, 3) for x in y] for y in eigen_crit]

    # Print results
    print(f'Johansen Test for cointegration between {pair.equity1.symbol} and ' 
          f'{pair.equity2.symbol} from {pair.start_date} to {pair.end_date}\n')

    for i in [0,1]:
        print(f'r<={i} Trace Statistic = {trace_stat[i]}\n'
              f'r<={i} Trace Critical Values:\n'
              f'90%: {trace_crit[i][0]}\n'
              f'95%: {trace_crit[i][1]}\n'
              f'99% {trace_crit[i][2]}')

    print('\n')

    for i in [0,1]:
        print(f'r<={i} Eigenvalue Statistic = {eigen_stat[i]}\n'
              f'r<={i} Eigenvalue Critical Values:\n'
              f'90%: {eigen_crit[i][0]}\n'
              f'95%: {eigen_crit[i][1]}\n'
              f'99% {eigen_crit[i][2]}')

    print('\n')



from datetime import datetime
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
import pandas as pd
import requests
from radium.helpers import _convert_date


class Equity:
    """
    Class for a single equity between two dates.

    Attributes
    ----------
    data : pd.DataFrame
        Contains all daily signals with date as index
    high : pd.Series
        Contains daily high prices with date as index
    low : pd.Series
        Contains daily low prices with date as index
    open : pd.Series
        Contains daily open prices with date as index
    closed : pd.Series
        Contains daily adjusted closed prices with date as index
    symbol : str
        Symbol for equity as found on exchange
    start_date : datetime.date
        First date of interest
    end_date : datetime.date
        Last date of interest
    key : str
        Alpha-vantage API key
    """

    def __init__(self, symbol, start_date, end_date, key):
        """
        Initialises equity class

        Parameters
        ----------
        symbol : str
            Symbol for equity as found on exchange
        start_date : str or datetime or datetime.date
            First date of interest in YYYY-MM-DD form
        end_date : str of datetime or datetime.date
            Last date of interest in YYYY-MM-DD form
        key : str
            Alpha-vantage API key

        Raises
        ------
        ValueError
            API Key is invalid
            Equity symbol does not exist
            End date is same as or before start date
        RuntimeError
            API Call limit reached
        """

        # Convert dates from strings to date objects
        start_date = _convert_date(start_date)
        end_date = _convert_date(end_date)

        # Raises error if date range invalid
        if end_date <= start_date:
            raise ValueError("end_date is the same as or before start_date")

        # Raises error if key is empty string
        if len(key) == 0:
            raise ValueError("Invalid API Key")

        self.symbol = symbol
        self.start_date = start_date
        self.end_date = end_date
        self.key = key

        # Fetch all data
        df = self._daily()

        # Get dates of interest only
        mask = (df.index >= start_date) & (df.index <= end_date)
        df = df.loc[mask]

        # Sort date earliest first
        df.sort_index(inplace=True)

        # Set data attribute
        self.data = df

        # Set each possible price type
        self.high = df["2. high"]
        self.low = df["3. low"]
        self.open = df["1. open"]
        self.closed = df["5. adjusted close"]

    def _daily(self):
        """
        Gets all available daily signals from an equity between two dates

        Contains open, high, low, close, adjusted close, volume, dividend
        amount, split coefficient, for each day/

        Returns
        -------
        ret : pd.DataFrame Dataframe containing daily signal
        information with date as an index, sorted most recent first.

        Raises
        ------
        ValueError
            Equity symbol does not exist
        RuntimeError
            API Call limit reached
        """

        # Get signals in JSON form
        url = f"https://www.alphavantage.co/query?function" \
              f"=TIME_SERIES_DAILY_ADJUSTED&symbol={self.symbol}" \
              f"&apikey={self.key}" \
              f"&outputsize=full"
        json = requests.get(url).json()

        # Extract signals
        try:
            # If correct data present place time series into list
            daily_json = json["Time Series (Daily)"]
        except KeyError:
            # Test whether an error message is recieved
            try:
                # If error message received from API, incorrect symbol used
                error_json = json["Error Message"]
                raise ValueError("Equity Symbol does not exist")
            except KeyError:
                # Otherwise call limit reached
                raise RuntimeError(
                    "API call limit reached, try again in 1 minute.")

        df = pd.DataFrame(daily_json).T

        # Format data as numerical
        columns = list(df.columns)
        for col in columns:
            df[col] = pd.to_numeric(df[col])

        # Format index as a date
        df.index = pd.to_datetime(df.index).date

        return df

    def plot(self, start_date=None, end_date=None):
        """
        Plots closed prices of equity between two dates as a line graph

        Parameters
        ----------
        start_date : (optional) str or datetime or datetime.date
            First date to plot in YYYY-MM-DD form, defaults to equity start date
        end_date : (optional) str of datetime or datetime.date
            Last date to plot in YYYY-MM-DD form, defaults to equity end date

        Raises
        ------
        ValueError
            End date is same as or before start date
        """

        # If no start/end date specified use default
        if start_date is None:
            start_date = self.start_date
        else:
            start_date = _convert_date(start_date)

        if end_date is None:
            end_date = self.end_date
        else:
            end_date = _convert_date(end_date)

        # Raises error if date range invalid
        if end_date <= start_date:
            raise ValueError("end_date is the same as or before start_date")

        # Gets required range only
        closed = self.closed
        mask = (closed.index >= start_date) & (closed.index <= end_date)
        closed = closed.loc[mask]

        fig, ax = plt.subplots()
        ax.plot(closed)

        plt.title(f"{self.symbol} from {start_date} to {end_date}")
        plt.xlabel("Date")
        plt.ylabel("Adjusted closed prices ($)")

        # Put dollar marks infront of y axis
        formatter = ticker.FormatStrFormatter('$%1.2f')
        ax.yaxis.set_major_formatter(formatter)

        plt.grid()
        plt.show()



import numpy as np
import pandas as pd

from radium import Pair


class PairStrategy:
    """
    Base class for equity pair trading strategies.

    Attributes
    ----------
    daily_returns : float np.ndarray[]
        Daily returns from trading by optimal positions
    cum_returns : float np.ndarray[]
        Daily cumulative returns from trading by optimal positions.
    CAGR : float
        Compound annual growth rate based upon daily cumulative returns.
    sharpe : float
        Sharpe ratio of the strategy.
    pair : radium.pair
    """

    def __init__(self, pair):
        """
        Initialises PairStrategy class

        Parameters
        ----------
        pair : radium.Pair

        Raises
        ------
        TypeError
            If pair isn't radium.Pair.
        """

        # Exception handling
        if not isinstance(pair, Pair):
            raise TypeError('pair must be an radium.Pair object')
        self.pair = pair

    @property
    def daily_returns(self):
        """
        float np.ndarray[] : Daily returns from trading by optimal positions

        Raises
        ------
        Exception
            If self.th_positions isn't defined
        """

        if hasattr(self, 'th_positions') == False:
            raise Exception('PairStrategy.th_positions is not defined')

        # Calculate th_daily_returns if undefined
        if hasattr(self, '_th_daily_returns') == False:
            # Get closed prices
            prices = pd.concat(
                [self.pair.equity1.closed, self.pair.equity2.closed], axis=1)

            # Calculate capital allocation to each position
            position_values = self.th_positions * prices.values

            # Convert positions_values to DataFrame
            position_values = pd.DataFrame(position_values,
                                           index=self.pair.equity1.data.index)
            position_values.columns = [self.pair.equity1.symbol,
                                       self.pair.equity2.symbol]

            # Calculate P&L with % change of close price and positions
            close_pct_change = prices.pct_change().values
            pnl = np.sum(position_values.shift().values * close_pct_change,
                         axis=1)

            # Calculate total position values the day before
            total_position_values = np.sum(np.abs(position_values.shift()),
                                           axis=1)
            self._th_daily_returns = pnl / total_position_values

        return self._th_daily_returns

    @property
    def cum_returns(self):
        """
        float np.ndarray[] : Daily cumulative returns from trading by optimal positions.

        Raises
        ------
        Exception
            If self.th_positions is not defined
        """

        if hasattr(self, 'th_positions') == False:
            raise Exception('PairStrategy.th_positions is not defined')

        # Calculate cum_returns if undefined
        if hasattr(self, '_cum_returns') == False:
            # TODO: Determine 252vs365 days
            # ret = self.daily_returns
            # cum_ret = pd.DataFrame((np.cumprod(1 + ret) - 1))
            # cum_ret.fillna(method='ffill', inplace=True)
            # self._cum_returns = cum_ret.to_numpy()

            self._cum_returns = np.cumprod(1 + self.daily_returns) - 1

        return self._cum_returns

    @property
    def CAGR(self):
        """
        float: Compound annual growth rate based upon daily cumulative returns.

        Raises
        ------
        Exception
            If self.th_positions is not defined.
        """

        if hasattr(self, 'th_positions') == False:
            raise Exception('PairStrategy.th_positions is not defined')

        # Calculate CAGR if undefined
        if hasattr(self, '_CAGR') == False:
            start_date = self.pair.start_date
            end_date = self.pair.end_date

            days = (end_date - start_date).days
            days = int(days)

            self._CAGR = (1 + self.cum_returns[-1]) ** (365 / days) - 1

        return self._CAGR

    @property
    def sharpe(self):
        """
        float: Sharpe ratio of the strategy.

        Measures the performance of an investment compared to a risk-free asset,
        after adjusting for its risk.

        Exception
            If self.th_positions is not defined
        """

        if hasattr(self, 'th_positions') == False:
            raise Exception('PairStrategy.th_positions is not defined')

        if hasattr(self, '_sharpe') == False:
            ret = self.daily_returns
            self._sharpe = np.sqrt(252) * np.mean(ret) / np.std(ret)

        return self._sharpe

# TODO: Research how to calculate.
#    @property
#     def max_drawdown(self):
#        """
#        Calculates the maximum drawdown of the investment.
#
#        The maximum observed loss from a peak to a trough of a portfolio, before
#        a new peak is attained.
#
#        Returns
#        -------
#        ret : float
#            Maximum drawdown
#
#        Raises
#        ------
#        Exception
#            If self.th_positions is not defined
#        """
#
#        if hasattr(self, '_th_positions') == False:
#            raise Exception('PairStrategy.th_positions is not defined')
#
#        if hasattr(self, '_max_drawdown') == False:
#            cum_ret = self.cum_returns
#            max_drawdown = (np.min(cum_ret) - np.max(cum_ret)) / np.max(cum_ret)
#            self._max_drawdown = max_drawdown
#
#     def MDD_duration(self):
#        """
#        Calculates duration between maximum drawdown peaks in days
#
#        Returns
#        -------
#        ret : int
#            Maximum drawdown duration
#        """
#
#        ret = self.th_returns()
#        max_drawdown_days = np.abs(np.argmax(ret) - np.argmax(min))
#        return max_drawdown_days



from datetime import datetime, date
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
import statsmodels.formula.api as sm

from radium import Equity
from radium.helpers import _truncate, _convert_date


class Pair:
    """
    Class for a pair of equities.

    Attributes
    ----------
    price_spread : float np.ndarray[]
        Price spread of equities for self.hedge_ratios
    equity1 : radium.Equity
    equity2 : radium.Equity
    start_date : datetime.date
    end_date : datetime.date
    hedge_ratios : float np.ndarray[][2]
        Day-wise pairs of hedge ratios

    """

    def __init__(self, equity1, equity2):
        """
        Initialise Pair class

        Parameters
        ----------
        equity1 : radium.Equity 
        equity2 : radium.Equity 

        Raises
        ------
        TypeError
            If equity1 or equity2 is not of type radium.Equity.
        ValueError
            If equity1 and equity2 do not share any date ranges
        """
        if not isinstance(equity1, Equity):
            raise TypeError('equity1 must of type radium.Equity')
        elif not isinstance(equity2, Equity):
            raise TypeError('equity2 must of type radium.Equity')

        self.equity1 = equity1
        self.equity2 = equity2

        # Sets start date as latest start date of equities
        if equity1.start_date >= equity2.start_date:
            self.start_date = equity1.start_date
        else:
            self.start_date = equity2.start_date

        # Sets end date as earliest end date of equities
        if equity1.end_date <= equity2.end_date:
            self.end_date = equity1.end_date
        else:
            self.end_date = equity2.end_date

        # If end date is before start date then date ranges of equities is
        # incompatible
        if self.end_date <= self.start_date:
            raise ValueError("There is no shared date ranges between equity1"
                             "and equity2")

    def hedge(self, method, lookback):
        """
        Calculates the hedge_ratios given a method and lookback and stores it
        in self.hedge_ratios

        Parameters
        ----------
        method : str
            Method for calculating hedge ratios ('ols')
        lookback: int
            Number of signals to lookback on when calculating hedge ratios

        Raises
        ------
        TypeError
            If method isn't a string.
            If lookback isn't an integer.
        ValueError
            If lookback <= 0.
            If method isn't available.

        Notes
        -----
        Available methods: 'OLS'
        """

        # Exception handling of method
        if not isinstance(method, str):
            raise TypeError('method must be a string')
        
        # Exception handling of lookback
        if not isinstance(lookback, int):
            raise TypeError('lookback must be an integer.')
        elif lookback <= 0:
            raise ValueError('lookback must be > 0')

        # Calculate hedge ratios based on the method provided
        if method == 'OLS':
            self.hedge_ratios = self._hedge_ols(lookback)
        else:
            raise ValueError('Available method strings: "OLS"')

    @property
    def price_spread(self):
        """
        float np.ndarray[] : Price spread of equities for self.hedge_ratios

        Raises
        ------
        TypeError
            If self.hedge_ratios isn't defined.

        Notes
        -----
        Spread calculated using y = h1*y1 + h2*y2.
        """

        if hasattr(self, 'hedge_ratios') == False:
            raise Exception('Pair.hedge_ratios is not defined.')

        # Calculate price_spread if undefined
        if hasattr(self, '_price_spread') == False:
            # Construct dataframe of closed prices
            prices = pd.concat([self.equity1.closed, self.equity2.closed],
                                axis=1)
            prices.columns = [self.equity1.symbol, self.equity2.symbol]

            # Multiply and add for each date
            self._price_spread = np.sum(self.hedge_ratios * prices, axis=1)

        return self._price_spread

    def budget(self, hedge_ratio, dec):
        """
        Calculates budget needed to buy integer number of equities.

        Parameters
        ----------
        hedge_ratio : int np.ndarray[2]
            Hedge ratios of pair
        dec : int
            Number of decimals to truncate to 

        Returns
        -------
        budget : float
            Budget needed rounded to 2 d.p.

        Raises
        -----
        TypeError
            If hedge_ratio isnt a list of floats, or dec isnt an integer.
        ValueError
            If hedge_ratio isn't length 2 or dec < 0.
        """

        if not isinstance(hedge_ratio, list):
            raise TypeError('hedge_ratio must be a list')
        elif not len(hedge_ratio) == 2:
            raise ValueError('hedge_ratio must have length 2')
        elif not isinstance(hedge_ratio[0], float):
            raise TypeError('hedge_ratio must be a list of floats')
        elif not isinstance(hedge_ratio[1], float):
            raise TypeError('hedge_ratio must be a list of floats')

        if not isinstance(dec, int):
            raise TypeError('Decimal places must be an integer.')
        elif dec < 0:
            raise ValueError('Decimal places has to be >= 0.')

        # Get the prices at end_date
        equity1_price = self.equity1.closed.iloc[-1]
        equity2_price = self.equity2.closed.iloc[-1]

        # Calculate truncated ratios to given number of decimals 
        truncated_ratios = np.array([_truncate(n, dec) for n in hedge_ratio])

        # Calculate budget to buy integer number of equites
        budget = equity1_price * abs(truncated_ratios[0]) * 10 ** dec \
                 + equity2_price * abs(truncated_ratios[1]) * 10 ** dec

        # Truncated budget to 2 decimals places
        budget = _truncate(budget, 2)

        return budget

    def plot_closed(self, start_date=None, end_date=None):
        """
        Plots closed prices of both equities between two dates as a line graph

        Parameters
        ----------
        start_date : (optional) str or datetime or datetime.date
            First date to plot in YYYY-MM-DD form, defaults to equity start date
        end_date : (optional) str of datetime or datetime.date
            Last date to plot in YYYY-MM-DD form, defaults to equity end date

        Raises
        ------
        ValueError
            End date is same as or before start date
        """

        # If no start/end date specified use default
        if start_date is None:
            start_date = self.start_date
        else:
            start_date = _convert_date(start_date)

        if end_date is None:
            end_date = self.end_date
        else:
            end_date = _convert_date(end_date)

        # Raises error if date range invalid
        if end_date <= start_date:
            raise ValueError('end_date is the same as or before start_date')
        elif start_date < self.start_date:
            raise ValueError('start_date can\'t be before pair.start_date')
        elif end_date > self.end_date:
            raise ValueError('end_date can\'t be after pair.end_date')

        # Gets required range only for both equities
        equity1_closed = self.equity1.closed
        mask = (equity1_closed.index >= start_date) \
               & (equity1_closed.index <= end_date)
        equity1_closed = equity1_closed.loc[mask]

        equity2_closed = self.equity2.closed
        mask = (equity2_closed.index >= start_date) \
               & (equity2_closed.index <= end_date)
        equity2_closed = equity2_closed.loc[mask]

        fig, ax = plt.subplots()
        plt.plot(equity1_closed, label=self.equity1.symbol)
        plt.plot(equity2_closed, label=self.equity2.symbol)

        title = (f'{self.equity1.symbol} and {self.equity2.symbol} '
                 f'from {start_date} to {end_date}')
        plt.title(title)
        plt.xlabel("Date")
        plt.ylabel("Adjusted closed prices ($)")

        # Put dollar marks infront of y axis
        formatter = ticker.FormatStrFormatter('$%1.2f')
        ax.yaxis.set_major_formatter(formatter)

        plt.legend()
        plt.grid()
        plt.show()

    def plot_price_spread(self):
        """
        Plots price spread of the pair given hedge_ratios

        Raises
        ------
        Exception
            If self.hedge_ratios is not defined.
        """

        if hasattr(self, '_hedge_ratios') == False:
            raise Exception('Pair.hedge_ratios is not defined.')

        dates = self.equity1.closed.index.values
        price_spread = self.price_spread

        plt.plot(dates, price_spread)

        title = f'{self.equity1.symbol} and {self.equity2.symbol} price spread'
        plt.title(title)
        plt.xlabel('Date')
        plt.ylabel('Price Spread ($)')

        plt.grid()
        plt.show()

    def _hedge_ols(self, lookback):
        """
        Calculate pair hedge ratios by OLS regression.

        self.equity1 will be used as response variable when regressing.

        Parameters
        ----------
        lookback : int
            Number of signals to lookback on when regressing.

        Returns 
        -------
        hedge_ratios : float np.ndarray[][2]
            Hedge ratios as [[1, -1*(OLS gradient)],...].
        """

        # Construct dataframe of closed prices
        df = pd.concat([self.equity1.closed, self.equity2.closed], axis=1)
        df.columns = [self.equity1.symbol, self.equity2.symbol]

        # Get ols regression result for each date
        hedge_ratios = np.zeros(df.shape)
        for i in range(lookback, hedge_ratios.shape[0]):
            formula = f"{self.equity1.symbol} ~ {self.equity2.symbol}"
            df_lookback = df[(i - lookback):i]
            ols = sm.ols(formula, df_lookback).fit()

            # Hedge ratio for equity2 is -1*(OLS gradient)
            hedge_ratios[i - 1][0] = 1
            hedge_ratios[i - 1][1] = -1*ols.params[1]

        return hedge_ratios



import pathlib
from setuptools import find_packages, setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# Get requirements
with open('requirements.txt') as f:
    required = f.read().splitlines()

# This call to setup() does all the work
setup(
    name="Radium-Tech",
    version="0.1.0",
    description="Intuitive backtesting for quantitative trading strategies.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/CallumRai/Radium-Tech",
    author="Callum Rai, Ivan Erlic",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    install_requires=required,
    entry_points={
        "console_scripts": [
            "realpython=reader.__main__:main",
        ]
    },
)
